@@ -52,7 +52,12 @@
 #define TOC_OFFSET 150
 #define GLADE_RIPPER_FILE "goobox.glade"
 #define DESTINATION_PERMISSIONS 0755
+#define PLS_PERMISSIONS 0644
 #define UPDATE_DELAY 400
+#define DEFAULT_OGG_QUALITY 0.3
+#define DEFAULT_FLAC_COMPRESSION 5
+#define DEFAULT_MP3_BITRATE 128
+#define BUFFER_SIZE 1024
 
 typedef struct {
 	GooWindow     *window;
@@ -63,8 +68,10 @@ typedef struct {
 	char          *artist;
 	char          *genre;
 	GList         *tracks;
+	int            tracks_n;
 	int            total_tracks;
 	GList         *current_track;
+	int            current_track_n;
 	char          *ext;
 	GooCdrom      *cdrom;
 
@@ -75,7 +82,9 @@ typedef struct {
 	GstPad        *source_pad;
 	GstFormat      track_format, sector_format;
 	guint          update_handle;
-	double         fraction;
+	int            total_sectors;
+	int            current_track_sectors;
+	int            prev_tracks_sectors;
 	char          *current_file;
 	gboolean       ripping;
 
@@ -92,8 +101,10 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	if (data->update_handle != 0)
+	if (data->update_handle != 0) {
 		g_source_remove (data->update_handle);
+		data->update_handle = 0;
+	}
 
 	if (data->ripping && (data->current_file != NULL))
 		gnome_vfs_unlink (data->current_file);
@@ -126,12 +137,18 @@ static gboolean
 rip_next_track (gpointer callback_data)
 {
 	DialogData *data = callback_data;
+	TrackInfo  *track;
 
 	g_free (data->current_file);
 	data->current_file = NULL;
 
 	gst_element_set_state (data->encoder, GST_STATE_NULL);
 	gst_element_set_state (data->rip_thread, GST_STATE_NULL);
+
+	track = data->current_track->data;
+	data->prev_tracks_sectors += track->sectors;
+
+	data->current_track_n++;
 	data->current_track = data->current_track->next;
 	rip_current_track (data);
 
@@ -143,7 +160,11 @@ static gboolean
 update_ui (gpointer callback_data)
 {
 	DialogData *data = callback_data;
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->r_progress_progressbar), data->fraction);
+	double      fraction;
+
+	fraction = ((double) (data->current_track_sectors + data->prev_tracks_sectors)) / (double) data->total_sectors;
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->r_progress_progressbar), fraction);
+
 	return FALSE;
 }
 
@@ -169,15 +190,17 @@ update_progress_cb (gpointer callback_data)
 	guint64      sector = 0;
 	gboolean     ret;
 
-	if (data->update_handle != 0)
+	if (data->update_handle != 0) {
 		g_source_remove (data->update_handle);
+		data->update_handle = 0;
+	}
 	
 	ret = gst_pad_query (data->source_pad, GST_QUERY_POSITION, &data->sector_format, &sector);
 	if (!ret)
 		return FALSE;
 	
 	from_sector = track->from_sector - TOC_OFFSET;
-	data->fraction = ((double) (sector - from_sector)) / (double)track->sectors;
+	data->current_track_sectors = sector - from_sector;
 	g_idle_add (update_ui, data);
 
 	data->update_handle = g_timeout_add (UPDATE_DELAY, update_progress_cb, data);
@@ -189,27 +212,41 @@ update_progress_cb (gpointer callback_data)
 static void
 create_pipeline (DialogData *data)
 {
-
+	float ogg_quality;
+	int   flac_compression;
+	int   mp3_bitrate;
 
 	data->rip_thread = gst_thread_new ("rip_thread");
 		
 	data->source = gst_element_factory_make ("cdparanoia", "cdparanoia");
 	g_object_set (G_OBJECT (data->source), 
-		      "location", data->location,
+		      "device", data->location,
 		      NULL);
 
 	switch (data->format) {
 	case GOO_FILE_FORMAT_OGG:	
 		data->encoder = gst_element_factory_make (OGG_ENCODER, "encoder");
 		data->ext = "ogg";
+		ogg_quality = eel_gconf_get_float (PREF_ENCODER_OGG_QUALITY, DEFAULT_OGG_QUALITY) + 0.05;
+		g_object_set (data->encoder,
+			      "quality", ogg_quality,
+			      NULL);
 		break;
 	case GOO_FILE_FORMAT_FLAC:
 		data->encoder = gst_element_factory_make (FLAC_ENCODER, "encoder");
 		data->ext = "flac";
+		flac_compression = eel_gconf_get_integer (PREF_ENCODER_FLAC_COMPRESSION, DEFAULT_FLAC_COMPRESSION);
+		g_object_set (data->encoder,
+			      "quality", flac_compression,
+			      NULL);
 		break;
 	case GOO_FILE_FORMAT_MP3:
 		data->encoder = gst_element_factory_make (MP3_ENCODER, "encoder");
 		data->ext = "mp3";
+		mp3_bitrate = eel_gconf_get_integer (PREF_ENCODER_MP3_BITRATE, DEFAULT_MP3_BITRATE);
+		g_object_set (data->encoder,
+			      "bitrate", mp3_bitrate,
+			      NULL);
 		break;
 	case GOO_FILE_FORMAT_WAVE:
 		data->encoder = gst_element_factory_make (WAVE_ENCODER, "encoder");
@@ -247,6 +284,126 @@ zero_padded (int n)
 }
 
 
+static char*
+get_destination_folder (DialogData *data)
+{
+	return g_build_filename (data->destination,
+				 data->artist,
+				 data->album,
+				 NULL);
+}
+
+
+static void
+done_dialog_response_cb (GtkDialog  *dialog,
+			 int         button_number,
+			 gpointer    userdata)
+{
+	DialogData *data = (DialogData*) userdata;
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	
+	if ((button_number == GTK_RESPONSE_OK)
+	    && eel_gconf_get_boolean (PREF_RIPPER_VIEW_DISTINATION, FALSE)) {
+		GError *error  = NULL;
+		char   *folder = get_destination_folder (data);
+		char   *url    = g_strconcat ("file://", folder, NULL);
+
+		if (! gnome_url_show (url, &error))
+			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->window), _("Could not display the destination folder"), &error);
+
+		g_free (folder);
+		g_free (url);
+	}
+
+	gtk_widget_destroy (data->dialog);
+}
+
+
+static void
+save_playlist (DialogData *data)
+{
+	char           *folder;
+	char           *filename;
+	GnomeVFSResult  result;
+	GnomeVFSHandle *handle;
+
+	folder = get_destination_folder (data);
+	filename = g_strconcat ("file://", folder, "/", data->album, ".pls", NULL);
+
+	gnome_vfs_unlink (filename);
+
+	result = gnome_vfs_create (&handle,
+				   filename,
+				   GNOME_VFS_OPEN_WRITE,
+				   TRUE,
+				   PLS_PERMISSIONS);
+
+	if (result == GNOME_VFS_OK) {
+		GList *scan;
+		char   buffer[BUFFER_SIZE];
+		int    n = 0;
+
+		strcpy (buffer, "[playlist]\n");
+		gnome_vfs_write (handle,
+				 buffer,
+				 strlen (buffer),
+				 NULL);
+
+		sprintf (buffer, "NumberOfEntries=%d\n", data->tracks_n);
+		gnome_vfs_write (handle,
+				 buffer,
+				 strlen (buffer),
+				 NULL);
+
+		strcpy (buffer, "Version=2\n");
+		gnome_vfs_write (handle,
+				 buffer,
+				 strlen (buffer),
+				 NULL);
+		
+		for (scan = data->tracks; scan; scan = scan->next) {
+			TrackInfo *track = scan->data;
+			char      *track_filename;
+			/*char      *track_path;*/
+
+			n++;
+
+			track_filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), track->title, data->ext);
+
+			/*
+			track_path = g_build_filename (folder, track_filename, NULL);
+			*/
+
+			sprintf (buffer, "File%d=%s\n", n, track_filename);
+			gnome_vfs_write (handle,
+					 buffer,
+					 strlen (buffer),
+					 NULL);
+			g_free (track_filename);
+			/*g_free (track_path);*/
+
+			sprintf (buffer, "Title%d=%s - %s\n", n, data->artist, track->title);
+			gnome_vfs_write (handle,
+					 buffer,
+					 strlen (buffer),
+					 NULL);
+
+			sprintf (buffer, "Length%d=%d\n", n, track->min * 60 + track->sec);
+			gnome_vfs_write (handle,
+					 buffer,
+					 strlen (buffer),
+					 NULL);
+		}
+
+		gnome_vfs_close (handle);
+	}
+
+	g_free (filename);
+	g_free (folder);
+}
+
+
 static void
 rip_current_track (DialogData *data)
 {
@@ -257,25 +414,44 @@ rip_current_track (DialogData *data)
 	GstEvent  *event;
 
 	if (data->current_track == NULL) {
+		GtkWidget *d;
+
+		if (eel_gconf_get_boolean (PREF_EXTRACT_SAVE_PLAYLIST, TRUE))
+			save_playlist (data);
+
 		data->ripping = FALSE;
-		gtk_widget_destroy (data->dialog);
+		gtk_widget_hide (data->dialog);
+
+		d = _gtk_ok_dialog_with_checkbutton_new (GTK_WINDOW (data->window),
+							 GTK_DIALOG_MODAL,
+							 _("Tracks extracted successfully"),
+							 GTK_STOCK_OK,
+							 _("_View destination folder"),
+							 PREF_RIPPER_VIEW_DISTINATION);
+							    
+		g_signal_connect (G_OBJECT (d), "response",
+				  G_CALLBACK (done_dialog_response_cb),
+				  data);
+		gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
+		gtk_widget_show (d);
+
 		return;
 	}
 		
 	track = data->current_track->data;
-	msg = g_strdup_printf (_("Extracting: %s"), track->title);
+	msg = g_strdup_printf (_("Extracting track %d of %d: %s"), 
+			       data->current_track_n,
+			       data->tracks_n,
+			       track->title);
 	gtk_label_set_text (GTK_LABEL (data->r_progress_label), msg);
 	g_free (msg);
 
 	/* Set the filename */
 
-	filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), track->title, data->ext);
-	folder = g_build_filename (data->destination,
-				   data->artist,
-				   data->album,
-				   NULL);
+	folder = get_destination_folder (data);
 	ensure_dir_exists (folder, DESTINATION_PERMISSIONS);
 
+	filename = g_strdup_printf ("%s - %s.%s", zero_padded (track->number + 1), track->title, data->ext);
 	g_free (data->current_file);
 	data->current_file = g_build_filename (folder, filename, NULL);
 	g_free (filename);
@@ -304,8 +480,7 @@ rip_current_track (DialogData *data)
 	/* Seek to track. */
 
 	gst_element_set_state (data->rip_thread, GST_STATE_PAUSED);
-	event = gst_event_new_segment_seek (data->track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, 
-					    track->number, track->number + 1);
+	event = gst_event_new_segment_seek (data->track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, track->number, track->number + 1);
 	if (!gst_pad_send_event (data->source_pad, event)) {
 		g_warning ("seek failed");
 		return;
@@ -313,6 +488,7 @@ rip_current_track (DialogData *data)
 
 	/* Start ripping. */
 
+	data->current_track_sectors = 0;
 	data->update_handle = g_timeout_add (UPDATE_DELAY, update_progress_cb, data);
 
 	gst_element_set_state (data->rip_thread, GST_STATE_PLAYING);
@@ -322,9 +498,23 @@ rip_current_track (DialogData *data)
 static void
 start_ripper (DialogData *data)
 {
-	goo_cdrom_lock_tray (data->cdrom);
+	GList *scan;
+
 	data->ripping = TRUE;
+
+	goo_cdrom_lock_tray (data->cdrom);
+
+	data->prev_tracks_sectors = 0;
+
+	data->total_sectors = 0;
+	for (scan = data->tracks; scan; scan = scan->next) {
+		TrackInfo *track = scan->data;
+		data->total_sectors += track->sectors;
+	}
+
 	create_pipeline (data);
+
+	data->current_track_n = 1;
 	data->current_track = data->tracks;
 	rip_current_track (data);
 }
@@ -364,6 +554,7 @@ dlg_ripper (GooWindow     *window,
 	data->artist = g_strdup (artist);
 	data->genre = g_strdup (genre);
 	data->tracks = track_list_dup (tracks);
+	data->tracks_n = g_list_length (data->tracks);
 	data->total_tracks = total_tracks;
 	data->update_handle = 0;
 
@@ -378,6 +569,11 @@ dlg_ripper (GooWindow     *window,
 
 	/* Set widgets data. */
 
+#ifdef HAVE_GTK_2_5
+	gtk_label_set_ellipsize (GTK_LABEL (data->r_progress_label),
+				 PANGO_ELLIPSIZE_END);
+#endif
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog), 
